/***************************************************************************
HTTP Response Data Structure Class
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.ghear;

/**  Simple data-structure class for HTTP responses.

@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: Response.java,v 1.1 2000/05/16 22:40:23 samuelg Exp $
*/
public class Response {
	public int returnCode;
	public String returnText;
	public boolean returnFile;
	
	/**
	Constructor.
	@param text the text of the message that's returned to the browser. It's
	automatically set to a 200 OK (HTTP_OK) HTTP status code.
	*/
	public Response(String text)
		{
		returnText = text;
		returnCode = 200;
		returnFile = false;
		}
		
	/**
	Constructor.
	@param retCode the HTTP Status code to be returned. You can get these out of HttpURLConnector,
	or manually enter the integer value.
	@param text the text of the message that's returned to the browser.
	*/	
	public Response(int retCode, String text)
		{
		returnText = text;
		returnCode = retCode;
		returnFile = false;
		}
	/**
	Constructor, with a sneaky way to return a file rather than a message.
	@param retCode the HTTP Status code to be returned. You can get these out of HttpURLConnector,
	or manually enter the integer value.
	@param filespec the specification of the file containing the message that's
	returned to the browser.
	@param isFile if this is set to true, the contents of the file will be returned
	to the browser. Otherwise, the filespec will be returned.
	*/	
	public Response(int retCode, String filespec, boolean isFile)
		{
		returnText = filespec;
		returnCode = retCode;
		returnFile = isFile;
		}
	
	/**
	@return an integer containing the HTTP status code.
	*/
	public int getCode()
		{
		return returnCode;
		}
		
	/**
	@return an integer containing the length of the text to be returned, or the
	length of the filespec if a file is to be returned
	*/
	public int getLen()
		{
		return returnText.length();
		}
	
	/**
	@return a String containing the text to be returned to the browser, or the
	name of the file to be returned
	*/
	public String getText()
		{
		return returnText;
		}
	
	/**
	@return a boolean. True if the response will return a file to the browser.
	*/
	public boolean isFile()
		{
		return returnFile;
		}
	}
