/***************************************************************************
Service Abstract Class.
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.ghear;

import java.util.*;
import java.net.*;
import com.andthehorseyourodeinon.utils.*;

/** This is an abstract class describing a Service.
A Service may be anything that runs in its own thread, and
does something. It needs a few methods to perform its reporting
duties for the GHEAR framework.

@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: Service.java,v 1.1 2000/05/16 22:41:13 samuelg Exp $
*/
public abstract class Service extends Thread {

	protected Properties options;
	protected Metrics mets;

	public Service()
		{
		options = new Properties();
		}

	/**
	@param options Service-level Properties (specific to the Services, not the Server)
	@param mets global Metrics object
	*/
	public Service(Properties options, Metrics mets)
		{
		this.options = options;
		this.mets = mets;
		}
	
	/** Override this with the main function of the Service
	*/
	public void run()
		{
		
		}
	
	/** Override this with something that returns a reasonable
	Service name
	*/
	public String getServiceName()
		{
		return "";
		}
	
	/** Override this with something that returns a Service Status String.
	*/
	public String getServiceStatus()
		{
		return "";
		}

	/* This is the portion that handles HTTP requests. The path of the
	   request is handed over as a string. It can be parsed however makes
	   sense.
	   
	   One example for simple, single-level requests would look like:
	   
		if (reportRequest.toLowerCase().startsWith("request1"))
			{
			return new Response(200,"This is a request for a report of type 1");
			}
		else if (reportRequest.toLowerCase().startsWith("request2"))
			{
			return new Response(200,"This is a request for a report of type 2");
			}
		else
			{
			return new Response(404,"request "+reportRequest+" not found.");
			}
	
	   so "http://server:port/request1" would return "This is a request for a report of type 1".
	   	
	*/ 
	public String getServiceReport(String reportRequest)
		{
		return "";
		}

}
