/***************************************************************************
Altavista Monitor Service
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.netgrep;

import java.util.*;
import java.net.*;
import com.andthehorseyourodeinon.utils.*;
import com.andthehorseyourodeinon.ghear.*;

/**  Implementation of a Search Engine Monitor Service that works
for the Altavista "Raging" Search Engine.
@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: AltavistaMonitorService.java,v 1.2 2000/05/22 04:53:43 samuelg Exp $
*/
public class AltavistaMonitorService extends SearchEngineMonitorService {

	public AltavistaMonitorService(Properties options, Metrics mets, String searchTerm, int depth)
		{
		super(options, mets, searchTerm, "http://ragingsearch.altavista.com/", "cgi-bin/query?q=",
			"&FFF=off&wfmt=tau&prf=Submit&Translate=off&nbq=", "", depth);
		}

	/** Altavista-specific implementation of Monitor refreshUrlList function
	*/
	public void refreshUrlList(boolean reportNewUrls)
		{
		theRequest = searchHost +
			 queryString + URLEncoder.encode(searchTerm) + postFix + depth;

		mets.start("Fetching Altavista Page");
 		String theDocList = HTTPUtils.getHTMLPage(theRequest, new Hashtable());
 		mets.stop("Fetching Altavista Page");

		mets.start("Altavista Monitor Parsing");		
		Vector theURLs = StringUtils.extractURLs(theDocList);

		Enumeration e = theURLs.elements();
		int counter = 0;
		while (e.hasMoreElements())
			{
			String thisUrl = (String)e.nextElement();
			if (!hasUrl(thisUrl))
				{
				if (reportNewUrls)
					{
					if (!options.getProperty("send","no").equals("no"))
						{
						sendUpdate("Search Engine Reference Found!",
							"New reference on Altavista is URL http://"+thisUrl+
							" has been found for search terms " + searchTerm);
						}
					Logger.debug(Logger.INFORM,"Adding new URL http://"+thisUrl);
					}
				hasNewUrls = true;
				urls.put(thisUrl, new SearchEngineRecord(thisUrl,counter));
				}
			counter++;
			}
		updateTimestamp();
		mets.stop("Altavista Monitor Parsing");		
		}

	public String getServiceName()
		{
		return "AltavistaMonitorService: "+searchTerm;
		}
		
	public String getServiceStatus()
		{
		return "OK";
		}
		
	public boolean isMonitor()
		{
		return true;
		}
		
	public String monitorTerms()
		{
		return searchTerm;
		}
}
