/***************************************************************************
Altavista Tracker Service
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.netgrep;

import java.util.*;
import java.net.*;
import com.andthehorseyourodeinon.utils.*;

/**  Implementation of a Search Engine Tracker Service that works
for the Altavista "Raging" Search Engine.
@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: AltavistaTrackerService.java,v 1.2 2000/05/22 04:53:44 samuelg Exp $
*/
public class AltavistaTrackerService extends SearchEngineMonitorService {

	private final static int VERYBIGNUMBER = Integer.MAX_VALUE;
	private Vector thePosition;
	private int bestPos;
	private final static String[] textvalue = { "<B>THE TOP!</B>","<B>SECOND!</B>","<B>THIRD!</B>",
			"Fourth!", "Fifth!", "sixth", "seventh", "eighth", "ninth", "tenth"};

	/**
	@param options Properties general to all the search engine Monitors and Trackers.
	@param mets Metrics for all the search engine Monitors and Trackers.
	@param searchTerm a String containing the search terms to use against the search engine.
	@param site the site to track position and position changes.
	@param depth the number of results from the search engine to consider.
	*/
	public AltavistaTrackerService(Properties options, Metrics mets,
		String searchTerm, String site, int depth)
		{
		super(options, mets, searchTerm, "http://ragingsearch.altavista.com/", "cgi-bin/query?q=",
			"&FFF=off&wfmt=tau&prf=Submit&Translate=off&nbq=", site, depth);
		}

	/** Altavista-specific implementation of Tracker refreshUrlList function
	*/
	public void refreshUrlList(boolean reportNewUrls)
		{
		if (thePosition == null)
			{
			thePosition = new Vector();
			}
		bestPos = VERYBIGNUMBER;
		theRequest = searchHost +
			 queryString + URLEncoder.encode(searchTerm) + postFix + depth;
		
		mets.start("Fetching Altavista Page");
		String theDocList = HTTPUtils.getHTMLPage(theRequest, new Hashtable());
		mets.stop("Fetching Altavista Page");
			
		if (theDocList == null)
			{
			System.out.println("!");
			}
		mets.start("Altavista Tracker Parsing");
		Vector theURLs = StringUtils.extractURLs(theDocList);
		Enumeration e = theURLs.elements();
		int counter = 0;
		boolean positionChanges = false;
		while (e.hasMoreElements())
			{
			String thisUrl = (String)e.nextElement();

			if (!hasUrl(thisUrl))
				{
				urls.put(thisUrl, new SearchEngineRecord(thisUrl,counter));
				}
				
			if (thisUrl.startsWith(soughtSite))
				{
				if (counter < bestPos)
					{
					bestPos = counter;
					}
					
				// check if it's in the list, and, if so, if it has moved.
				SearchEngineRecord thisRec = (SearchEngineRecord)urls.get(thisUrl);
				if ((thisRec != null) && (thisRec.getPosition() != counter))
					{
					thisRec.setPosition(counter);
					positionChanges = true;
					if (reportNewUrls)
						{
						if (!options.getProperty("send","no").equals("no"))
							{
							sendUpdate("Search Engine Position Change!",
								"URL http://"+thisUrl+" has moved from "+
								thisRec.getPreviousPosition()+" to "+ thisRec.getPosition() +
								" for search terms " + searchTerm + " on Altavista");
							}
						Logger.debug(Logger.INFORM,"URL http://"+thisUrl+" has moved from "+
							thisRec.getPreviousPosition()+" to "+ thisRec.getPosition() + " for search terms "+
							searchTerm + " on Altavista");
						}
					}
				}
			counter++;
			}
		if ((positionChanges) || (firstTime))
			{
			addToHistory();
			}
		if (theDocList != null)
			{
			firstTime = false;
			}
		updateTimestamp();
		mets.stop("Altavista Tracker Parsing");
		}

	/** Add a record to the URL/Site position history
	*/
	public void addToHistory()
		{
		String record;
		if ((bestPos >= 0) && (bestPos < 10))
			{
			record = new String("<TR><TD>"+StringUtils.createCompactDateString() +
				"</TD><TD>" + textvalue[bestPos] + "</TD></TR>\n");
			}
		else if (bestPos == VERYBIGNUMBER)
			{
			record = new String("<TR><TD>"+StringUtils.createCompactDateString() +
				"</TD><TD> (not listed) </TD></TR>\n");
			}
		else
			{
			record = new String("<TR><TD>"+StringUtils.createCompactDateString() +
				"</TD><TD>" + (bestPos + 1) + "</TD></TR>\n");
			}
		thePosition.addElement(record);
		}

	public String getSiteString()
		{
		return soughtSite;
		}

	/**
	@return String describing site/URL's current position on Altavista
	for the search terms.
	*/
	public String currentPosition()
		{
			
		if (bestPos == VERYBIGNUMBER)
			{
			return "Not Listed";
			}
		else if (bestPos < 10)
			{
			return textvalue[bestPos];
			}
		else
			{
			return Integer.toString(bestPos+1);
			}
		}

	/**
	@return String describing site/URL's position history on Altavista
	 for the search terms.
	*/
	public String history()
		{
		StringBuffer sb = new StringBuffer();
		sb.append("<TR><TD COLSPAN=\"2\" ALIGN=\"CENTER\">Position Change History</TD></TR>\n");
		sb.append("<TR><TD>Date</TD><TD>Position</TD></TR>\n");
		Enumeration e = thePosition.elements();
		while (e.hasMoreElements())
			{
			sb.append((String)e.nextElement());
			}
		return sb.toString();
		}

	public String getServiceName()
		{
		return "AltavistaTrackerService for "+soughtSite+" using "+searchTerm;
		}
		
	public String getServiceStatus()
		{
		return "OK";
		}
		
	public boolean isMonitor()
		{
		return false;
		}		
}
