/***************************************************************************
Google Monitor Service
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.netgrep;

import java.util.*;
import java.net.*;
import com.andthehorseyourodeinon.utils.*;
import com.andthehorseyourodeinon.ghear.*;

/**  Implementation of a Search Engine Monitor Service that works
for the Google Search Engine.
@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: GoogleMonitorService.java,v 1.2 2000/05/22 04:53:44 samuelg Exp $
*/
public class GoogleMonitorService extends SearchEngineMonitorService {

	/**
	@param options Properties general to all the search engine Monitors and Trackers.
	@param mets Metrics for all the search engine Monitors and Trackers.
	@param searchTerm a String containing the search terms to use against the search engine.
	@param depth the number of results from the search engine to consider.
	*/
	public GoogleMonitorService(Properties options, Metrics mets, String searchTerm, int depth)
		{
		super(options, mets, searchTerm, "http://www.google.com/", "search?q=",
			"&meta=hl%3Den%26lr%3D&btnG=Google+Search&num=", "", depth);
		}

	/** Google-specific implementation of Monitor refreshUrlList function
	*/
	public void refreshUrlList(boolean reportNewUrls)
		{
		theRequest = searchHost +
			 queryString + URLEncoder.encode(searchTerm) + postFix + depth;

		mets.start("Fetching Google Page");
 		String theDocList = HTTPUtils.getHTMLPage(theRequest, new Hashtable());
 		mets.stop("Fetching Google Page");

		mets.start("Google Monitor Parsing");		
		Vector theURLs = StringUtils.extractURLs(theDocList);

		Enumeration e = theURLs.elements();
		int counter = 0;
		while (e.hasMoreElements())
			{
			String thisUrl = (String)e.nextElement();
			if (!hasUrl(thisUrl))
				{
				if (reportNewUrls)
					{
					if (!options.getProperty("send","no").equals("no"))
						{
						sendUpdate("Search Engine Reference Found!",
							"New reference on Google is URL http://"+thisUrl+
							" has been found for search terms " + searchTerm);
						}
					Logger.debug(Logger.INFORM,"Adding new URL http://"+thisUrl);
					}
				hasNewUrls = true;
				urls.put(thisUrl, new SearchEngineRecord(thisUrl,counter));
				}
			counter++;
			}
		updateTimestamp();
		mets.stop("Google Monitor Parsing");		
		}

	public String getServiceName()
		{
		return "GoogleMonitorService: "+searchTerm;
		}
		
	public String getServiceStatus()
		{
		return "OK";
		}
		
	public boolean isMonitor()
		{
		return true;
		}
		
	public String monitorTerms()
		{
		return searchTerm;
		}
}
