/***************************************************************************
NetGrep Service Dispatcher
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.netgrep;

import java.io.*;
import java.net.*;
import java.util.*;
import com.andthehorseyourodeinon.utils.*;
import com.andthehorseyourodeinon.ghear.*;

/**  Implementation of a Service Dispatcher that works
for the Search Engine Trackers and Monitors. It's fairly straight-forward,
but contains a lot of HTML that should probably be templatized.
@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: NetGrepServiceDispatcher.java,v 1.2 2000/05/22 04:53:44 samuelg Exp $
*/
public class NetGrepServiceDispatcher extends com.andthehorseyourodeinon.ghear.ServiceDispatcher {

	public NetGrepServiceDispatcher()
		{
		services = new Hashtable();
		dispatcherName = "NetGrep Dispatcher";
		}

	public void initDispatcher(Properties serverOptions, Metrics metrics)
		{
		this.serverOptions = serverOptions;
		this.mets = metrics;
		metrics.start("Initialize NetGrep Dispatcher");
		Logger.log("Starting NetGrepper version 1.1.");
		options = new Properties();
		try
			{
			options.load(new FileInputStream("netgrep.conf"));
			}
		catch (IOException fnfe)
			{
			Logger.log("Can't read properties from netgrep.conf");
			}
		Logger.log("NetGrepper version 1.1. initialized");
		metrics.stop("Initialize NetGrep Dispatcher");
		}
		
	public void initServices()
		{
		mets.start(" Initialization of NetGrepServiceDispatcher");
		try
			{
 			BufferedReader termFile =
				new BufferedReader(new FileReader(options.getProperty("termFileSpec","searchterms")));


			/* Read in the serach term file line by line, parsing and creating SearchEngineMonitors as
			   we go. This is somewhat inelegant, and could benefit from being converted into some kind
			   of XML-like config file. Laziness wins out, though. */
			String line = "";
			int counter = 0;
			StringTokenizer theTokens;
			while ((line = termFile.readLine()) != null)
				{
				if ((line.length() > 0) && (line.charAt(0) != '#'))
					{
					String engine = "google";
					String terms = "";
					int depth = 10;
					String site = "";
					theTokens = new StringTokenizer(line,"|");
					boolean parsing = theTokens.hasMoreTokens();
					counter++;
					while (parsing)
						{
						String thisToken = theTokens.nextToken();
						if (thisToken.indexOf("=") == -1)
							{
							Logger.debug(Logger.ERROR,"Can't understand search directive: "+line);
							parsing = false;
							}
						else
							{
							if (thisToken.regionMatches(true,0,"engine",0,"engine".length()))
								{
								engine = thisToken.substring(thisToken.indexOf("=")+1).toLowerCase();
								}
							else if (thisToken.regionMatches(true,0,"term",0,"term".length()))
								{
								terms = thisToken.substring(thisToken.indexOf("=")+1);
								}
							else if (thisToken.regionMatches(true,0,"site",0,"site".length()))
								{
								site = thisToken.substring(thisToken.indexOf("=")+1);
								if (StringUtils.indexOfIgnoreCase(site,"http://") == 0)
									{
									site = site.substring("http://".length());
									}
								if (site.endsWith("/"))
									{
									site = site.substring(0,site.length()-1);
									}
								}
							else if (thisToken.regionMatches(true,0,"depth",0,"depth".length()))
								{
								depth = Integer.parseInt(thisToken.substring(thisToken.indexOf("=")+1));
								}
							}
						if (parsing)
							{
							parsing = theTokens.hasMoreTokens();
							}
						}
					if (site == "")
						{
						Logger.debug(Logger.BORE,"Creating a "+engine+" monitor search for "+terms);
						if (engine.equals("google"))
							{
							addService(Integer.toString(counter),
								new GoogleMonitorService(options, mets, terms, depth));
							}
						else if (engine.equals("altavista"))
							{
							addService(Integer.toString(counter),
								new AltavistaMonitorService(options, mets, terms, depth));
							
							}
						}
					else
						{
						Logger.debug(Logger.BORE,"Creating a "+engine+" position track for "+site+" using "+terms);
						
						if (engine.equals("google"))
							{
							addService(Integer.toString(counter),
								new GoogleTrackerService(options, mets, terms, site, depth));
							}
						else if (engine.equals("altavista"))
							{
							addService(Integer.toString(counter),
								new AltavistaTrackerService(options, mets, terms, site, depth));
							
							}
						}
					}
				}
			}
		catch (Exception e)
			{
			Logger.debug(Logger.ERROR,"Failure reading file "+options.getProperty("termFileSpec","searchterms"));
			Logger.debug(Logger.ERROR,"Exception: "+e.getMessage());
			e.printStackTrace();
			}

		Logger.debug(Logger.INFORM,"Started a total of "+getServiceCount()+" Services.");
		mets.stop(" Initialization of NetGrepServiceDispatcher");
		}
		


	public Response getDispatchReport(ParameterVector pv)
		{
		if (((String)pv.stringAt(1)).toLowerCase().startsWith("/monitors"))
			{
			StringBuffer s = new StringBuffer();
			Enumeration e = services.keys();
			s.append("<TABLE BORDER=\"1\"><TD COLSPAN=\"2\"><B>Currently Active Search Engine Monitors</B></TD></TR>\n");
			s.append("<TR><TD ALIGN=\"CENTER\"><B>ID</B></TD><TD ALIGN=\"CENTER\"><B>Task</B></TD></TR>\n");
			while (e.hasMoreElements())
				{
				String theID = (String)e.nextElement();
				SearchEngineMonitorService sems = (SearchEngineMonitorService)services.get(theID);
				if (sems.isMonitor())
					{
					s.append("<TR><TD ALIGN=\"CENTER\"><A HREF=\"/monitor=");
					s.append(theID);
					s.append("\">");
					s.append(theID);
					s.append("</A></TD><TD>"+sems.getServiceName()+"</TD></TR>\n");
					}
				}
			s.append("</TABLE>\n");
			return new Response(HttpURLConnection.HTTP_OK,s.toString());
			}
		else if (((String)pv.stringAt(1)).toLowerCase().startsWith("/trackers"))
			{
			StringBuffer s = new StringBuffer();
			Enumeration e = services.keys();
			s.append("<TABLE BORDER=\"1\"><TD COLSPAN=\"2\"><B>Currently Active Search Engine Site Trackers</B></TD></TR>\n");
			s.append("<TR><TD ALIGN=\"CENTER\"><B>ID</B></TD><TD ALIGN=\"CENTER\"><B>Task</B></TD></TR>\n");
			while (e.hasMoreElements())
				{
				String theID = (String)e.nextElement();
				SearchEngineMonitorService sems = (SearchEngineMonitorService)services.get(theID);
				if (!sems.isMonitor())
					{
					s.append("<TR><TD ALIGN=\"CENTER\"><A HREF=\"/tracker=");
					s.append(theID);
					s.append("\">");
					s.append(theID);
					s.append("</A></TD><TD>"+sems.getServiceName()+"</TD></TR>\n");
					}
				}
			s.append("</TABLE>\n");
			return new Response(HttpURLConnection.HTTP_OK,s.toString());
			}
		else if (((String)pv.stringAt(1)).toLowerCase().startsWith("/monitor="))
			{
			StringBuffer s = new StringBuffer();
			String theID = ((String)pv.stringAt(1)).substring(((String)pv.stringAt(1)).indexOf("=")+1);
			SearchEngineMonitorService sems = (SearchEngineMonitorService)services.get(theID);
			if (sems == null)
				{
				return new Response(HttpURLConnection.HTTP_NOT_FOUND,
					"Monitor ID: "+theID+" not found!");
				}
			s.append("<TABLE BORDER=\"1\"><TD COLSPAN=\"2\"><B>Search Engine Monitor Status</B></TD></TR>\n");
			s.append("<TR><TD>Search Engine:</TD><TD>");
			s.append(sems.getSearchHost());
			s.append("</TD></TR>\n");
			s.append("<TR><TD>Search Terms:</TD><TD>");
			s.append(sems.getSearchString());
			s.append("</TD></TR>\n");
			s.append("<TR><TD>Search Depth:</TD><TD>");
			s.append(sems.getSearchDepth());
			s.append(" <A HREF=\"/monitorresults=");
			s.append(theID);
			s.append("\">(found ");
			s.append(sems.getResultCount());
			s.append(")");
			s.append(" results</A></TD></TR>\n");
			s.append("<TR><TD>Query Issued:</TD><TD><FONT SIZE=\"-2\">");
			s.append(sems.getRequest());
			s.append("</FONT></TD></TR>\n");
			s.append("<TR><TD>Update Frequency:</TD><TD>");
			s.append(StringUtils.reasonableTimeUnits(
				Long.parseLong(options.getProperty("refresh", "86400000"))));
			s.append("</TD></TR>\n");
			s.append("<TR><TD>Last Updated:</TD><TD>");
			s.append(StringUtils.toRFC1123DateString(sems.getTimestamp()));
			s.append("</TD></TR>\n");
			s.append("<TR><TD>New URLs?:</TD><TD>");
			s.append(sems.hasNewUrl());
			s.append("</TD></TR>\n");
			s.append("</TABLE>\n");
			return new Response(HttpURLConnection.HTTP_OK,s.toString());
			}
		else if (((String)pv.stringAt(1)).toLowerCase().startsWith("/tracker="))
			{
			StringBuffer s = new StringBuffer();
			String theID = ((String)pv.stringAt(1)).substring(((String)pv.stringAt(1)).indexOf("=")+1);
			SearchEngineMonitorService sems = (SearchEngineMonitorService)services.get(theID);
			if (sems == null)
				{
				return new Response(HttpURLConnection.HTTP_NOT_FOUND,
					"Tracker ID: "+theID+" not found!");
				}
			s.append("<TABLE BORDER=\"1\"><TD COLSPAN=\"2\"><B>Search Engine Site Tracker Status</B></TD></TR>\n");
			s.append("<TR><TD>Search Engine:</TD><TD>");
			s.append(sems.getSearchHost());
			s.append("</TD></TR>\n");
			s.append("<TR><TD>Site Tracked:</TD><TD>");
			s.append(sems.getSiteString());
			s.append("</TD></TR>\n");
			s.append("<TR><TD>Current Position:</TD><TD> ");
			s.append(sems.currentPosition());
			s.append(" <A HREF=\"/trackhist=");
			s.append(theID);
			s.append("\">view history</A></TD></TR>\n");
			s.append("<TR><TD>Search Terms:</TD><TD>");
			s.append(sems.getSearchString());
			s.append("</TD></TR>\n");
			s.append("<TR><TD>Search Depth:</TD><TD>");
			s.append(sems.getSearchDepth());
			s.append(" <A HREF=\"/monitorresults=");
			s.append(theID);
			s.append("\">(found ");
			s.append(sems.getResultCount());
			s.append(")");
			s.append(" results</TD></TR>\n");
			s.append("<TR><TD>Query Issued:</TD><TD><FONT SIZE=\"-2\">");
			s.append(sems.getRequest());
			s.append("</FONT></TD></TR>\n");
			s.append("<TR><TD>Update Frequency:</TD><TD>");
			s.append(StringUtils.reasonableTimeUnits(
				Long.parseLong(options.getProperty("refresh", "86400000"))));
			s.append("</TD></TR>\n");
			s.append("<TR><TD>Last Updated:</TD><TD>");
			s.append(StringUtils.toRFC1123DateString(sems.getTimestamp()));
			s.append("</TD></TR>\n");
			s.append("</TABLE>\n");
			return new Response(HttpURLConnection.HTTP_OK,s.toString());
			}
		else if (((String)pv.stringAt(1)).toLowerCase().startsWith("/monitorresults="))
			{
			StringBuffer s = new StringBuffer();
			String theID = ((String)pv.stringAt(1)).substring(((String)pv.stringAt(1)).indexOf("=")+1);
			SearchEngineMonitorService sems = (SearchEngineMonitorService)services.get(theID);
			if (sems == null)
				{
				return new Response(HttpURLConnection.HTTP_NOT_FOUND,
					"Result Set for Monitor ID: "+theID+" not found!");
				}
			s.append("<TABLE BORDER=\"1\"><TD COLSPAN=\"3\"><B>URLs in Result Set</B></TD></TR>\n");
			s.append("<TR><TD COLSPAN=\"2\">Search Engine:</TD><TD>");
			s.append(sems.getSearchHost());
			s.append("</TD></TR>\n");
			s.append("<TR><TD COLSPAN=\"2\">Search Terms:</TD><TD>");
			s.append(sems.getSearchString());
			s.append("</TD></TR>\n");
			s.append("<TR><TD COLSPAN=\"2\">Search Depth:</TD><TD>");
			s.append(sems.getSearchDepth());
			s.append(" (found ");
			s.append(sems.getResultCount());
			s.append(")</TD></TR>\n");
			s.append("<TR><TD COLSPAN=\"2\">Update Frequency:</TD><TD>");
			s.append(StringUtils.reasonableTimeUnits(
				Long.parseLong(options.getProperty("refresh", "86400000"))));
			s.append("</TD></TR>\n");
			s.append("<TR><TD COLSPAN=\"2\">Last Updated:</TD><TD>");
			s.append(StringUtils.toRFC1123DateString(sems.getTimestamp()));
			s.append("</TD></TR>\n");
			Vector urls = sems.getUrls();
			Enumeration en = urls.elements();
			while (en.hasMoreElements())
				{
				SearchEngineRecord ser = (SearchEngineRecord)en.nextElement();
				s.append("<TR><TD>");
				s.append(ser.getPosition());
				s.append("</TD><TD>");
				s.append(StringUtils.createCompactDateString(ser.lastUpdated()));
				s.append("</TD><TD><A HREF=\"http://");
				s.append(ser.getUrl());
				s.append("\" TARGET=_new>http://");
				s.append(ser.getUrl());
				s.append("</A></TD></TR>");
				}
			s.append("</TABLE>\n");
			return new Response(HttpURLConnection.HTTP_OK,s.toString());
			}
		else if (((String)pv.stringAt(1)).toLowerCase().startsWith("/trackhist="))
			{
			StringBuffer s = new StringBuffer();
			String theID = ((String)pv.stringAt(1)).substring(((String)pv.stringAt(1)).indexOf("=")+1);
			SearchEngineMonitorService sems = (SearchEngineMonitorService)services.get(theID);
			if (sems == null)
				{
				return new Response(HttpURLConnection.HTTP_NOT_FOUND,
					"Result Set for Monitor ID: "+theID+" not found!");
				}
			s.append("<TABLE BORDER=\"1\"><TD COLSPAN=\"2\"><B>Tracker Position History</B></TD></TR>\n");
			s.append("<TR><TD>Search Engine:</TD><TD>");
			s.append(sems.getSearchHost());
			s.append("</TD></TR>\n");
			s.append("<TR><TD>Site Tracked:</TD><TD>");
			s.append(sems.getSiteString());
			s.append("</TD></TR>\n");
			s.append("<TR><TD>Current Position:</TD><TD> ");
			s.append(sems.currentPosition());
			s.append("</TD></TR>\n");
			s.append("<TR><TD>Search Terms:</TD><TD>");
			s.append(sems.getSearchString());
			s.append("</TD></TR>\n");
			s.append("<TR><TD>Search Depth:</TD><TD>");
			s.append(sems.getSearchDepth());
			s.append(" (found ");
			s.append(sems.getResultCount());
			s.append(")");
			s.append(" results</TD></TR>\n");
			s.append("<TR><TD>Update Frequency:</TD><TD>");
			s.append(StringUtils.reasonableTimeUnits(
				Long.parseLong(options.getProperty("refresh", "86400000"))));
			s.append("</TD></TR>\n");
			s.append("<TR><TD>Last Updated:</TD><TD>");
			s.append(StringUtils.toRFC1123DateString(sems.getTimestamp()));
			s.append("</TD></TR>\n");
			s.append(sems.history());
			s.append("</TABLE>\n");
			return new Response(HttpURLConnection.HTTP_OK,s.toString());
			}
		else
			{
			return new Response(HttpURLConnection.HTTP_NOT_FOUND,
				FileCache.toString("readme.html"));
			}
		}

}

