import java.util.*;
import java.net.*;
import com.andthehorseyourodeinon.utils.StringUtils;
import com.andthehorseyourodeinon.utils.HTTPUtils;
import com.andthehorseyourodeinon.utils.Logger;

public abstract class SearchEngineMonitor extends Thread {

	protected String searchTerm;
	protected String searchHost;
	protected String queryString;
	protected String postFix;
	protected int depth;
	protected Hashtable urls;
	protected boolean hasNewUrls = false;
	protected long updateFrequency;
	protected Calendar lastAccessed;
	protected Properties options;

	public SearchEngineMonitor(Properties options, String searchTerm, String searchHost, String queryString,
		String postFix, int depth)
		{
		urls = new Hashtable();
		this.searchTerm = searchTerm;
		this.searchHost = searchHost;
		this.queryString = queryString;
		this.postFix = postFix;
		this.options = options;
		this.depth = depth;
		lastAccessed = Calendar.getInstance();
		updateFrequency = Integer.parseInt(options.getProperty("refresh", "86400000")); /* run an update daily by default */
		Logger.debug(Logger.BORE,"Starting new thread looking for "+searchTerm);
		getUrlList();
		start();
		}
	
	public void run()
		{
		while (true)
			{
			Logger.debug(Logger.BORE,"Thread for "+searchTerm+" on "+searchHost+" awake.");
			refreshUrlList();

			try
				{
				sleep(updateFrequency);
				}
			catch (InterruptedException ie)
				{
				Logger.debug(Logger.ERROR,"Thread for "+searchTerm+" on "+searchHost+
					" interrupted!");
				}
			}
		}
		

	public void getUrlList()
		{
		refreshUrlList(false);
		}
		
	public void refreshUrlList()
		{
		refreshUrlList(true);
		}
	
	public void refreshUrlList(boolean reportNewUrls)
		{
		}

	public boolean hasUrl(String theUrl)
		{
		if (urls.containsKey(theUrl))
			{
			return true;
			}
		return false;
		}
		
	public SearchEngineRecord getUrl(String theUrl)
		{
		if (hasUrl(theUrl))
			{
			return (SearchEngineRecord)urls.get(theUrl);
			}
		else
			{
			return null;
			}
		}

	public Vector getUrlsModifiedAfter(Calendar cal)
		{
		Vector ret = new Vector();
		Enumeration e = urls.keys();
		while (e.hasMoreElements())
			{
			SearchEngineRecord theUrl = (SearchEngineRecord)e.nextElement();
			if ((theUrl.lastUpdated()).after(cal))
				{
				ret.addElement(theUrl);
				}
			}
		return ret;
		}

	public Vector getUrlsCreatedAfter(Calendar cal)
		{
		Vector ret = new Vector();
		Enumeration e = urls.keys();
		while (e.hasMoreElements())
			{
			SearchEngineRecord theUrl = (SearchEngineRecord)e.nextElement();
			if ((theUrl.created()).after(cal))
				{
				ret.addElement(theUrl);
				}
			}
		return ret;
		}
				
	public void updateTimestamp()
		{
		lastAccessed = Calendar.getInstance();
		}
		
	public Calendar getTimestamp()
		{
		return lastAccessed;
		}
		
	public boolean hasNewUrl()
		{
		boolean hadNewUrls = hasNewUrls;
		hasNewUrls = false;
		return hadNewUrls;
		}
		
	public String getSearchString()
		{
		return searchTerm;
		}

	public String getSearchHost()
		{
		return searchHost;
		}

}
