/***************************************************************************
File Manipulation Utility Functions.
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.utils;

import java.lang.*;
import java.io.*;
import java.util.*;

/** FileCache is a statically-called utility class that caches text files
for various uses. It tries to be intelligent about loading files when needed.
@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: FileCache.java,v 1.1 2000/05/16 22:45:10 samuelg Exp $
*/
public class FileCache implements Runnable {

	private static Hashtable forms= new Hashtable();
	private static Hashtable timestamp = new Hashtable();
	
	private FileCache()
		{
		new Thread(this).start();
		}
	
	/**
	Activate this as a single thread that won't get garbage collected.
	*/
	public void run()
		{
		try
			{
			wait();
			}
		catch (InterruptedException e)
			{
			}
		}

	/* returns the length of the specified file. If the file is not in
	the cache, it attempts to read it.
	@param filespec file identifier
	@return length of the contents of the file as represented by a String
	*/
	public static int length(String filespec)
		{
		if (inCache(filespec))
			{
			return ((String)forms.get(filespec)).length();
			}
		else
			{
			return 0;
			}
		}
		
	/* returns the contents of the specified file as a String. If the file is not in
	the cache, it attempts to read it.
	@param filespec file identifier
	@return the contents of the file as represented by a String
	*/
	public static String toString(String filespec)
		{
		if (inCache(filespec))
			{
			return (String)forms.get(filespec);
			}
		else
			{
			return "File Error";
			}
		}
		
	/** determines if the specified file is in the cache. If the file is not in the
	cache, it attempts to load the file so it can return "true."
	@param filespec file identifier
	@return true if the file is in the cache.
	*/
	public static boolean inCache(String filespec)
		{
		if (!forms.containsKey(filespec))
			{
			return loadFile(filespec);
			}
		File tf = new File(filespec);
		if (tf.lastModified() != ((Long)timestamp.get(filespec)).longValue())
			{
			return loadFile(filespec);
			}
		return true;
		}
			
	/** Attempt to load a file into the cache.
	@param filespec file identifier
	@return true if the file was successfully added to the cache.
	*/
	public static boolean loadFile(String filespec)
		{
		StringBuffer sb = new StringBuffer();
		Logger.debug(Logger.BORE,"Loading file "+filespec);
		try
			{
			BufferedReader inFile = new BufferedReader(
				new FileReader(filespec));

			boolean reading = true;
			String line;
			while (reading)
				{
				line = inFile.readLine();
				if (line == null)
					{
					reading = false;
					}
				else
					{
					sb.append(line);
					}
				}
			File tf = new File(filespec);
			timestamp.put(filespec,new Long(tf.lastModified()));
			}
		catch (Exception e)
			{
			return false;
			}
			
		forms.put(filespec,sb.toString());
		
		return true;
		}


}
