/***************************************************************************
HTTP Utility Functions.
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.utils;

import java.lang.*;
import java.util.*;
import java.text.DateFormat;
import java.net.*;
import java.io.*;
import java.text.DecimalFormat;

/** HTTPUtils is a statically-called utility class that contains useful functions
for HTTP-related activity. Some of these are pretty badly written, and are
only included for legacy reasons. Caveat Emptor!

@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: HTTPUtils.java,v 1.1 2000/05/16 22:45:10 samuelg Exp $
*/
public class HTTPUtils {
	/* number of characters that comprise a carriage-return line feed */
	public final static int CRLFLEN	= 2;

	/** converts an HTTP status code into its corresponding text message <I>e.g.</I>,
	statusMessage(200) returns "OK", or statusMessage(404) returns "Not Found".
	These <I>should</I> be RFC 2068 and RFC 2616 compliant.
	@param statusCode an integer HTTP status code
	@return String containing status message
	*/
	public static String statusMessage(int statusCode)
		{
		if (statusCode == HttpURLConnection.HTTP_ACCEPTED)
			{
			return "Accepted";
			}
  		if (statusCode == HttpURLConnection.HTTP_BAD_GATEWAY)
			{
			return "Bad Gateway";
			}
 		if (statusCode == HttpURLConnection.HTTP_BAD_METHOD )
			{
			return "Method Not Allowed";
			}
  		if (statusCode == HttpURLConnection.HTTP_BAD_REQUEST)
			{
			return "Bad Request";
			} 
 		if (statusCode == HttpURLConnection.HTTP_CLIENT_TIMEOUT)
			{
			return "Request Timeout";
			} 
  		if (statusCode == HttpURLConnection.HTTP_CONFLICT)
			{
			return "Conflict";
			} 
  		if (statusCode == HttpURLConnection.HTTP_CREATED)
			{
			return "Created";
			} 
  		if (statusCode == HttpURLConnection.HTTP_ENTITY_TOO_LARGE)
			{
			return "Request Entity Too Large";
			} 
		if (statusCode == HttpURLConnection.HTTP_FORBIDDEN)
			{
			return "Forbidden";
			} 
		if (statusCode == HttpURLConnection.HTTP_GATEWAY_TIMEOUT)
			{
			return "Gateway Timeout";
			} 
		if (statusCode == HttpURLConnection.HTTP_GONE)
			{
			return "Gone";
			} 
		if (statusCode == HttpURLConnection.HTTP_INTERNAL_ERROR)
			{
			return "Internal Server Error";
			} 
		if (statusCode == HttpURLConnection.HTTP_LENGTH_REQUIRED)
			{
			return "Length Required";
			} 
		if (statusCode == HttpURLConnection.HTTP_MOVED_PERM)
			{
			return "Moved Permanently";
			} 
		if (statusCode == HttpURLConnection.HTTP_MOVED_TEMP)
			{
			return "Temporary Redirect";
			} 
		if (statusCode == HttpURLConnection.HTTP_MULT_CHOICE)
			{
			return "Multiple Choices";
			} 
		if (statusCode == HttpURLConnection.HTTP_NO_CONTENT)
			{
			return "No Content";
			} 
		if (statusCode == HttpURLConnection.HTTP_NOT_ACCEPTABLE)
			{
			return "Not Acceptable";
			} 
		if (statusCode == HttpURLConnection.HTTP_NOT_AUTHORITATIVE)
			{
			return "Non-Authoritative Information";
			} 
		if (statusCode == HttpURLConnection.HTTP_NOT_FOUND)
			{
			return "Not Found";
			} 
		if (statusCode == HttpURLConnection.HTTP_NOT_MODIFIED)
			{
			return "Not Modified";
			} 
		if (statusCode == HttpURLConnection.HTTP_OK)
			{
			return "OK";
			} 
		if (statusCode == HttpURLConnection.HTTP_PARTIAL)
			{
			return "Partial Content";
			} 
		if (statusCode == HttpURLConnection.HTTP_PAYMENT_REQUIRED)
			{
			return "Payment Required";
			} 
		if (statusCode == HttpURLConnection.HTTP_PRECON_FAILED)
			{
			return "Precondition Failed";
			} 
		if (statusCode == HttpURLConnection.HTTP_PROXY_AUTH)
			{
			return "Proxy Authentication Required";
			}
		if (statusCode == HttpURLConnection.HTTP_REQ_TOO_LONG)
			{
			return "Request-URI Too Long";
			} 
		if (statusCode == HttpURLConnection.HTTP_RESET)
			{
			return "Reset Content";
			} 
		if (statusCode == HttpURLConnection.HTTP_SEE_OTHER)
			{
			return "See Other";
			} 
		if (statusCode == HttpURLConnection.HTTP_SERVER_ERROR)
			{
			return "Internal Server Error";
			} 
		if (statusCode == HttpURLConnection.HTTP_UNAUTHORIZED)
			{
			return "Unauthorized";
			} 
		if (statusCode == HttpURLConnection.HTTP_UNAVAILABLE)
			{
			return "Service Unavailable";
			} 
		if (statusCode == HttpURLConnection.HTTP_UNSUPPORTED_TYPE)
			{
			return "Unsupported Media Type";
			} 
		if (statusCode == HttpURLConnection.HTTP_USE_PROXY)
			{
			return "Use Proxy";
			} 
		if (statusCode == HttpURLConnection.HTTP_VERSION)
			{
			return "HTTP Version Not Supported";
			}
		
		// And the ones that aren't represented in HttpURLConnection?
		
		if (statusCode == 100)
			{
			return "Continue";
			}
		if (statusCode == 101)
			{
			return "Switching Protocols";
			}
		if (statusCode == 302)
			{
			return "Found";
			}
		if (statusCode == 416)
			{
			return "Requested Range Not Satisfiable";
			}
		if (statusCode == 417)
			{
			return "Expectation Failed";
			}
		if (statusCode == 501)
			{
			return "Not Implemented";
			}
			
		return "Unknown HTTP Code";
		}


	/** This function uses up a fair amount of network traffic to determine whether
	or not a given host is willing to serve up a general document via HTTP.
	@param theAddress the InetAddress of the server to test.
	@param thePort the port to talk to (for Web servers, this will generally be port 80).
	@return true if the host will serve that document to this client.
	*/
	public static boolean acceptingConnections(InetAddress theAddress, int thePort)
		{
		HttpURLConnection theConnection;
		
		try
			{
			return(acceptingConnections(new URL("http://"+
				theAddress.getHostName() + ":" + thePort)));
			}
		catch (MalformedURLException mfu)
			{
			return false;
			}
		}

	/** This function uses up a fair amount of network traffic to determine whether
	or not a given host is willing to serve up a given document.
	@param theAddress the InetAddress of the server to test.
	@param thePort the port to talk to (for Web servers, this will generally be port 80)
	@param theRequest the relative path and name of the document to test for on the specified host.
	@return true if the host will serve that document to this client.
	*/
	public static boolean acceptingConnections(InetAddress theAddress, int thePort, String theRequest)
		{
		HttpURLConnection theConnection;
		
		try
			{
			return(acceptingConnections(new URL("http://"+
				theAddress.getHostName() + ":" + thePort + theRequest)));
			}
		catch (MalformedURLException mfu)
			{
			return false;
			}
		}

	/** This function uses up a fair amount of network traffic to determine whether
	or not a given host is willing to serve up a given document.
	@param theUrl the URL of the document to test for.
	@return true if the host will serve that document to this client.
	*/
	public static boolean acceptingConnections(URL theUrl)
		{
		HttpURLConnection theConnection;
		
		try
			{
			theConnection = (HttpURLConnection)theUrl.openConnection();
			int retCode = theConnection.getResponseCode();
			theConnection.disconnect();
			if ((retCode == 200)||(retCode == 202))
				{
				return true;
				}
			}
		catch (IOException ioe)
			{
			return false;
			}
			
		return false;
		}


	/** This allows the processing of a HTML Form (or the equivalent) as submitted
	by a typical web browser. It is still somewhat rudimentary, although it has been
	successfully tested using Netscape 4.7 and Internet Explorer 4 and 5.
	It can be used for standard HTML forms, as well as file upload or other multipart
	forms.
	It identifies which kind of form has been submitted, and calls dispatches the request
	to either the singlePartForm or multiPartForm method.
	@param br A BufferedReader that contains the HTML Form submission.
	@return a PostedForm structure of the form contents.
	@see PostedForm
	*/
	public static PostedForm formSubmission(BufferedReader br)
		{
		PostedForm ret = new PostedForm();
		// read BR until we have a content encoding
		String theLine = new String();
		boolean gotAllHeaders = false;
		try
			{
			while (gotAllHeaders == false)
				{
				theLine = br.readLine().trim();
				if (theLine.length() == 0)
					{
					gotAllHeaders = true;
					}
				else
					{
					ret.addHeader(theLine);
					}
				}
			if (ret.getHeader("content-type").equalsIgnoreCase("application/x-www-form-urlencoded"))
				{
				// got a single-part FORM submission
				Logger.debug(Logger.BORE,"Handling a single-part form.");
				singlePartForm(ret, br);
				}
			else if (ret.getHeader("content-type").toLowerCase().indexOf("multipart/form-data") != -1)
				{
				// got a multi-part FORM submission
				Logger.debug(Logger.BORE,"Handling a multi-part form.");
				multiPartForm(ret,br);
				}
			else
				{
				// unknown encoding
				Logger.debug(Logger.ERROR,"Unknown FORM encoding!");
				}
			}
		catch (IOException ioe)
			{
			Logger.debug(Logger.ERROR,"Error handling submission.");
			}
		return ret;
		}

	/** 
	Handles a multi-part Form submission. Contents are placed into the PostedForm.
	Yuck!
	All this conversion back and forth hurts!
	There's gotta be a better way...
	@param p a PostedForm to use as a data store.
	@param br the BufferedReader through which the submission is received.
	*/
	public static void multiPartForm(PostedForm p, BufferedReader br)
		{
		int contentLength =0;
		try
			{
			contentLength = Integer.parseInt(p.getHeader("content-length"));
			char theContent[] = new char[contentLength];
			
			int read = 0;
			while (read != contentLength)
				{
				// loop until we can read the whole thing...
				// presumably, a broken pipe will trigger an IOException here, so
				// we're OK.
				read += br.read(theContent,read,contentLength-read);
				}
				
			// we have a big bunch of chars. We need to iterate across it
			// breaking it into chunks based on the boundary

			char[] boundary = p.getHeader("content-type").substring(
				p.getHeader("content-type").indexOf("boundary=")+9).trim().toCharArray();
			
			int boundPointer = 0;
			int contentPointer = 0;
			int bodyPointer = 0;
			int checkPointer = 0;
			boolean seekingBoundary = true;
			boolean foundPossibleBoundary = false;
			boolean firstBoundary = true;
			
			while (bodyPointer < contentLength)
				{
				if ((seekingBoundary) && 
					(bodyPointer < contentLength+1) &&
					(theContent[bodyPointer] == '-') &&
					(theContent[bodyPointer+1] == '-'))
					{
					seekingBoundary = false;
					foundPossibleBoundary = true;
					bodyPointer += CRLFLEN;
					boundPointer=0;
					continue;
					}
					
				if (foundPossibleBoundary)
					{
					if ((theContent[bodyPointer] == boundary[boundPointer]) &&
						(boundPointer == boundary.length-1))
						{
						// found a boundary!
						bodyPointer++;
						if (firstBoundary)
							{
							firstBoundary = false;
							checkPointer = bodyPointer;
							}
						else
							{
							// process this last section!
							
							
							// 1. identify the content type
							// 1a. find the first carriage return
							int crPointer = checkPointer;
							
							// get to the start of a new line
							while ((theContent[crPointer] == '\r') ||
								(theContent[crPointer] == '\n'))
								{
								crPointer++;
								checkPointer++;
								}
								
							// find the end of that line
							while ((crPointer < theContent.length) &&
								(theContent[crPointer] != '\r') &&
								(theContent[crPointer] != '\n'))
								{
								crPointer++;
								}
							// 1b. Get the first line following the boundary
							String sectionType = new String(theContent,checkPointer,
								crPointer-checkPointer);
							
							// 1c. extract the name
							String theName;
							if (sectionType.toLowerCase().indexOf("name=\"") != -1)
								{
								int nameStart = sectionType.toLowerCase().indexOf("name=\"")+6;
								theName = sectionType.substring(nameStart,
									sectionType.indexOf("\"",nameStart));
								}
							else
								{
								theName = "undefined";
								}
							
							// 1d. Is there another line of information? Like an encoding type?
							crPointer++;
							crPointer++;
							String theEncoding = null;
							if ((theContent[crPointer] != '\r') &&
								(theContent[crPointer] != '\n'))
								{
								// grab this next line.
								int nextCrPointer = crPointer;
								while ((nextCrPointer < theContent.length) &&
									(theContent[nextCrPointer] != '\r') &&
									(theContent[nextCrPointer] != '\n'))
									{
									nextCrPointer++;
									}
								
								theEncoding = new String(theContent,crPointer,
									nextCrPointer-crPointer);
								crPointer = nextCrPointer + CRLFLEN;
								}
							crPointer++;
							crPointer++;
					
							/* Ouch! I should find a way to pass a sub-array, rather than
							do all this ridiculous conversion! */
							String theContentArea = new String(theContent,crPointer,
								bodyPointer-crPointer-boundary.length- (2*CRLFLEN));
															
							if (theEncoding != null)
								{
								p.addContent(theName,theContentArea.getBytes(), theEncoding);
								}
							else
								{
								p.addContent(theName,theContentArea.getBytes());
								}							
							}
							
							
						boundPointer = 0;
						foundPossibleBoundary = false;
						seekingBoundary = true;
						checkPointer = bodyPointer;

						}
					else if ((theContent[bodyPointer] == boundary[boundPointer]) &&
						(boundPointer < boundary.length))
						{
						// going through a sequence that may be a boundary
						bodyPointer++;
						boundPointer++;
						continue;
						}
					else
						{
						// thought it was a boundary, but it's a bust
						boundPointer = 0;
						foundPossibleBoundary = false;
						seekingBoundary = true;
						continue;
						}
					}
				else
					{
					bodyPointer++;
					}
				}
			
			}
		catch (Exception e)
			{
			Logger.debug(Logger.ERROR,"Error handling FORM submission.");
			}
		}


	/** 
	Handles a single-part Form submission. Contents are placed into the PostedForm.
	@param p a PostedForm to use as a data store.
	@param br the BufferedReader through which the submission is received.
	*/		
	public static void singlePartForm(PostedForm p, BufferedReader br)
		{
		int contentLength = 0;
		try
			{
			contentLength = Integer.parseInt(p.getHeader("content-length"));
			char theContent[] = new char[contentLength]; 
			if (br.read(theContent,0,contentLength) != contentLength)
				{
				Logger.debug(Logger.ERROR,"content length read problem.");
				}
			
			String theContentString = new String(theContent);
			StringTokenizer st = new StringTokenizer(theContentString,"&\n");
			while (st.hasMoreTokens())
				{
				String thisContent = st.nextToken();
				if (thisContent.indexOf("=") != -1)
					{
					p.addContent(
						thisContent.substring(0,thisContent.indexOf("=")),
						(URLDecoder(
						thisContent.substring(
						thisContent.indexOf("=")+1))).getBytes());
					}
				}
			}
		catch (Exception e)
			{
			Logger.debug(Logger.ERROR,"Error handling submission.");
			}
		}

		
	/** The antithesis to URLEncoder.encode().
	This may be a bit weak, but it seems to work on simple application/x-www-url-encoded strings.
	@param content the String to decode
	@return the decoded String
	*/
	public static String URLDecoder(String content)
		{
		StringBuffer sb = new StringBuffer();
		int i=0;
		while (i<content.length())
			{
			if (content.charAt(i) == '+')
				{
				sb.append(" ");
				i++;
				}
			else if (content.charAt(i) == '%')
				{
				if (i < (content.length() - 2))
					{
					String theHex = ""+content.charAt(i+1)+content.charAt(i+2);
					Character theChar = new Character((char)Integer.parseInt(theHex,16));
					sb.append(theChar.toString());
					i += 3;
					}
				else
					{
					sb.append("%");
					i++;
					}
				}
			else
				{
				sb.append(content.charAt(i));
				i++;
				}
			
			}
		return sb.toString();
		}

	/**
	Given a URL and an optional collection of cookies, this will build a connection
	to that URL and prepare an InputStream to enable reading of the results. There is some general
	lameness here -- the routine doesn't allow you to specify your own headers, nor pass more than
	one cookie.
	@param urlString the URL to connect to.
	@param cookies a Hashtable containing any cookie information that you want to pass to
	the server, as a collection of name-value pairs.
	@return an InputStream from which you can read the response of the URL.
	*/
	public static InputStream getHTMLStream(String urlString, Hashtable cookies)
		{
		String retStr = new String();

		URL theUrl;
		HttpURLConnection theConnection;
		String cookiestring = new String();
		boolean first = true;
		
		if (cookies != null)
			{
			Enumeration headItr = cookies.keys();
			while (headItr.hasMoreElements())
				{
				if (first)
					{
					first = false;
					}
				else
					{
					cookiestring = cookiestring + "; ";
					}
				String head = (String)headItr.nextElement();
				cookiestring = cookiestring + head+"="+
					(String)cookies.get(head);
				}
			}


		Hashtable headers = new Hashtable();

		// Will I go to hell for making people's web logs interesting?
		headers.put("Connection","Keep-Alive");
		//headers.put("User-Agent","Mozilla/0.01 [en] (TRS-80/Compatible; NEWDOS/2.0; Z80)");
		headers.put("Accept","image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png, */*");
		headers.put("Accept-Language","en");
		headers.put("Accept-Charset","iso-8859-1,*,utf-8");
		if (cookiestring.length() > 0)
			{
			headers.put("Cookie",cookiestring);
			}
		
		try {
			theUrl = new URL(urlString);
			Logger.debug(Logger.BORE,"Attempting to open: "+urlString);
			theConnection = (HttpURLConnection)theUrl.openConnection();

			Enumeration headItr = headers.keys();
			while (headItr.hasMoreElements())
				{
				String head = (String)headItr.nextElement();
				theConnection.setRequestProperty(head,(String)headers.get(head));
				}

			theConnection.setDefaultAllowUserInteraction(false);
			theConnection.setDoOutput(false);
			theConnection.setDoInput(true);
			theConnection.setUseCaches(false);
			theConnection.setRequestMethod("GET");
			theConnection.setFollowRedirects(true);
			return theConnection.getInputStream(); 
			}
		catch (Exception e)
			{
			e.printStackTrace();
			retStr = "getHTMLPage Exception! GET "+urlString+ " caused exception "+e.getMessage();
			}
		return null;
		}


	/**
	Given a URL and an optional collection of cookies, this will build a connection
	to that URL and prepare an InputStream to enable reading of the results. There is some general
	lameness here -- the routine doesn't allow you to specify your own headers, nor pass more than
	one cookie.
	@param urlString the URL to connect to.
	@param cookies a Hashtable containing any cookie information that you want to pass to
	the server, as a collection of name-value pairs.
	@return a String containing the response of the URL.
	*/
	public static String getHTMLPage(String urlString, Hashtable cookies)
		{
		String retStr = new String();
	
		//Logger.debug(Logger.BORE,"Getting "+urlString);
		InputStream htmlIn = getHTMLStream(urlString, cookies);
		if (htmlIn == null)
			{
			//Logger.debug(Logger.BORE,"Got null responsse.");
			return null;
			}

		try 
			{
			BufferedReader in = new BufferedReader(new InputStreamReader(htmlIn)); 
			String s;
			while ((s=in.readLine()) != null)
				{
				retStr = retStr + s+ "\n";
				}
			in.close();
			}
		catch (Exception e)
			{
			e.printStackTrace();
			retStr = "getHTMLPage Exception! GET "+urlString+ " caused exception "+e.getMessage();
			}
		//Logger.debug(Logger.BORE,"Got "+retStr);
		return retStr;
		}


	/**
	A primitive, socket-based approach to an HTTP GET. 
	There is a great deal of
	uncruftiness here -- the routine doesn't allow you to specify your own headers, and is generally
	pretty narrowly implemented. Because it tries to follow redirects, this routine will recurse. The
	cut-off of five redirects is pretty arbitrary.
	@param urlString the URL to connect to.
	@param cookies a Vector containing prepared Cookie header fields.
	@param depth number of times we've recursed. You'll generally want to call it with 0.
	@return a String containing the response of the URL.
	*/
	public static String getHTMLPageRaw(String urlString, Vector cookieList, int depth)
		{
		String retStr = new String();

		URL theUrl;
		if (depth > 5)
			{
			return "Too many redirects...";
			}

		try {
			URL url = new URL(urlString);
			String theHost = url.getHost();
			int thePort = url.getPort();
			if (thePort == -1)
				{
				thePort = 80;
				}
			String thePath = url.getFile();

			InetAddress theHostIP = InetAddress.getByName(theHost);
			Socket socket = new Socket(theHostIP,thePort);
			PrintWriter out = new PrintWriter(new BufferedWriter(
				new OutputStreamWriter(socket.getOutputStream())),true);
			BufferedReader in = new BufferedReader (new InputStreamReader(
				socket.getInputStream()));
				
			// Once again, we mess with people's Web logs, and scare Richard Stallman. Why? Because we can!
			out.print("GET "+thePath+" HTTP/1.0\r\n");
			out.print("User-Agent: Mozilla/4.0 (compatible; MSIE 5.0; X11; I; MicrosoftLinux 2.2.15 i686)\r\n");
			out.print("Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png, */*\r\n");
			out.print("Accept-Encoding: gzip\r\n");
			out.print("Accept-Language: en\r\n");
			Enumeration cookies = cookieList.elements();
			while (cookies.hasMoreElements())
				{
				out.print("Cookie: "+cookies.nextElement()+"\r\n");
				}
			out.print("Accept-Charset: iso-8859-1,*,utf-8\r\n\r\n");
			out.flush();

			String s;
			while ((s=in.readLine()) != null)
				{
				retStr = retStr + s +"\n";
				}
			
			in.close();
			out.close();
			if (retStr.length() < 1)
				{
				// zero length results? This is an error!
				return null;
				}
			if (retStr.indexOf("HTTP/1.0 302") != -1)
				{
				// redirect! Handle it before returning.
				// is this specific to Reel.com?
				// whatever.
				Vector newCookies = new Vector();
				String dest="";
				StringTokenizer tokenizer=new StringTokenizer(retStr,"\n");
				while (tokenizer.hasMoreTokens())
					{
					String tok = tokenizer.nextToken();
					if (tok.indexOf(":") == -1)
						{
						// not a tagged line; ignore it.
						System.err.println("Ignoring it");
						continue;
						}
					if (tok.substring(0,tok.indexOf(":")).equalsIgnoreCase("Set-Cookie"))
						{
						newCookies.addElement(tok.substring(tok.indexOf(": ")+2));
						}
					else if (tok.substring(0,tok.indexOf(":")).equalsIgnoreCase("Location"))
						{
						dest = tok.substring(tok.indexOf(": ")+2);
						}
					}
				return getHTMLPageRaw(dest,newCookies,depth+1);
				}
			return retStr;
			}
		catch (Exception e)
			{
			e.printStackTrace();
			retStr = "httpPost Exception! POST "+urlString+ " caused exception "+e.getMessage();
			}
		return retStr;
		}

	/**
	A primitive, socket-based approach to an HTTP GET. 
	There is a great deal of
	uncruftiness here -- the routine doesn't allow you to specify your own headers, and is generally
	pretty narrowly implemented.
	@param urlString the URL to connect to.
	@param cookies a Vector containing prepared Cookie header fields.
	@return an InputStream from which you can read the response of the URL.
	*/
	public static InputStream getHTMLStreamRaw(String urlString, Vector cookieList)
		{
		return getHTMLStreamRaw(urlString, cookieList, null);
		}

	/**
	A primitive, socket-based approach to an HTTP GET. 
	There is a great deal of
	uncruftiness here -- the routine doesn't allow you to specify your own headers, and is generally
	pretty narrowly implemented. This also has a proxy parameter that will allow communication via an
	HTTP proxy.
	@param urlString the URL to connect to.
	@param cookies a Vector containing prepared Cookie header fields.
	@param proxy a HTTP proxy address
	@return an InputStream from which you can read the response of the URL.
	*/
	public static InputStream getHTMLStreamRaw(String urlString, Vector cookieList, String proxy)
		{
		String retStr = new String();
		try {
			URL url = new URL(proxy == null ? urlString : proxy);
			String theHost = url.getHost();
			int thePort = url.getPort();
			if (thePort == -1)
				{
				thePort = 80;
				}
			String thePath = proxy==null ? url.getFile() : urlString;

			InetAddress theHostIP = InetAddress.getByName(theHost);
			Socket socket = new Socket(theHostIP,thePort);
			PrintWriter out = new PrintWriter(new BufferedWriter(
				new OutputStreamWriter(socket.getOutputStream())),true);
			InputStream result = socket.getInputStream();
			
			// Do we never tire of this? Heck, I still put "Kick me" signs on my little brother.
			out.print("GET "+thePath+" HTTP/1.0\r\n");
			out.print("User-Agent: Mozilla/1.0 [en] (X11; I; Linux 1.0.2 i386)\r\n");
			out.print("Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png, */*\r\n");
			out.print("Accept-Encoding: gzip\r\n");
			out.print("Accept-Language: en\r\n");
			if (cookieList != null)
				{
				Enumeration cookies = cookieList.elements();
				while (cookies.hasMoreElements())
					{
					out.print("Cookie: "+cookies.nextElement()+"\r\n");
					}
				}
			out.print("Accept-Charset: iso-8859-1,*,utf-8\r\n\r\n");
			out.flush();
			return result;
			}
		catch (Exception e)
			{
			e.printStackTrace();
			retStr = "httpPost Exception! POST "+urlString+ " caused exception "+e.getMessage();
			}
		return null;
		}

	/**
	A primitive, socket-based approach to an HTTP GET, which then dumps to a file. 
	There is a great deal of
	uncruftiness here -- the routine doesn't allow you to specify your own headers, and is generally
	pretty narrowly implemented. It also has a proxy parameter that will allow communication via an
	HTTP proxy.
	@param urlString the URL to connect to.
	@param cookies a Vector containing prepared Cookie header fields.
	@param fileName the file to dump the response into
	@param proxy a HTTP proxy address
	@return a boolean indicating success or failure.
	*/
	public static boolean downloadHTMLPageRaw(String urlString, Vector cookies, String fileName, String proxy)
		{
		FileOutputStream out;
		InputStream in;

		try 
			{
			out = new FileOutputStream(fileName);
			}
		catch (Exception e)
			{
			System.err.println("Unable to open file " + fileName +": " + e);
			return false;
			}

		in = getHTMLStreamRaw(urlString, cookies, proxy);
		if (in == null)
			return false;

		try
			{
			// strip off headers
			int count=0;
			while(true)
				{
				int ch = in.read();
				if (ch<0)
					break;
				if ((char)ch == '\n') 
					{
					if (count<=2)
						break;
					count = 0;
					}
				count++;
				}

			copyStream(in, out);
			in.close();
			out.close();
			}
		catch (IOException e)
			{
			System.err.println("Unable to download file " + urlString +" to file " + fileName +"\n" + e);
			return false;
			}


		return true;
		}

	/**
	A routine to copy an InputStream to and OutputStream.
	@param in the source InputStream
	@param out the destination OutputStream
	*/
	public static void copyStream(InputStream in, OutputStream out) throws IOException
		{
		final int BUFFERSIZE = 4096*4;
		byte [] buffer = new byte[BUFFERSIZE];
		int numberRead;

		while ((numberRead = in.read(buffer)) >= 0)
			out.write(buffer, 0, numberRead);
		
		}



	/**
	Given a URL and encoded data, this will build a connection
	to that URL and perform an HTTP POST of that data. There is some general
	lameness here -- the routine doesn't allow you to specify your own headers.
	@param urlString the URL to connect to.
	@param urlEncodedData whatever it is that you want to pass to
	the server, as a collection of name-value pairs.
	@return a String containing the response from the server.
	*/
	public static String httpPost(String urlString, String urlEncodedData)
		{
		String retStr = new String();

		URL theUrl;
		HttpURLConnection theConnection;
		String cookiestring = new String();
		boolean first = true;
				
		try {
			URL url = new URL(urlString);
			URLConnection con = url.openConnection();
			con.setDoInput(true);
			con.setDoOutput(true);
			con.setUseCaches(false);
			// eh, enough already...
			con.setRequestProperty("User-Agent","Mozilla/4.07 [en] (X11; I; Linux 2.0.36 i686)");
			con.setRequestProperty("Content-Type","application/x-www-form-urlencoded");
			con.setRequestProperty("Content-Length",Integer.toString(urlEncodedData.length()));			
			con.setRequestProperty("Accept","image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png, */*");
			con.setRequestProperty("Accept-Encoding","gzip");
			con.setRequestProperty("Accept-Language","en");
			con.setRequestProperty("Accept-Charset","iso-8859-1,*,utf-8");
			
			DataOutputStream out = new DataOutputStream(con.getOutputStream());
			out.writeBytes(urlEncodedData);
			out.flush();
			out.close();
			

			BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream())); 
			String s;
			while ((s=in.readLine()) != null)
				{
				retStr = retStr + s;
				}
			
			in.close();
			}
		catch (Exception e)
			{
			e.printStackTrace();
			retStr = "httpPost Exception! POST "+urlString+ " caused exception "+e.getMessage();
			}
		return retStr;
		}

	/**
	A primitive, socket-based approach to an HTTP POST. 
	This is indeed
	uncrufty -- the routine doesn't allow you to specify your own headers, allows only a single cookie,
	and is generally pretty narrowly implemented.
	@param urlString the URL to connect to.
	@param urlEncodedData whatever it is that you want to pass to
	the server, as a collection of name-value pairs.
	@param cookie you can pass a single cookie along as a String
	@return a String containing the response from the server.
	*/
	public static String httpPostRaw(String urlString, String urlEncodedData, String cookie)
		{
		String retStr = new String();

		URL theUrl;

		try {
			URL url = new URL(urlString);
			// but we'll cheat...
			String theHost = url.getHost();
			int thePort = url.getPort();
			if (thePort == -1)
				{
				thePort = 80;
				}
			String thePath = url.getFile();

			InetAddress theHostIP = InetAddress.getByName(theHost);
			Socket socket = new Socket(theHostIP,thePort);
			PrintWriter out = new PrintWriter(new BufferedWriter(
				new OutputStreamWriter(socket.getOutputStream())),true);
			BufferedReader in = new BufferedReader (new InputStreamReader(
				socket.getInputStream()));
				
			// what? Why are you looking here?
			out.print("POST "+thePath+" HTTP/1.0\r\n");
			out.print("User-Agent: Mozilla/4.07 [en] (X11; I; Linux 2.0.36 i686)\r\n");
			out.print("Content-Type: application/x-www-form-urlencoded\r\n");
			out.print("Content-Length: " + Integer.toString(urlEncodedData.length()) +"\r\n");
			out.print("Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png, */*\r\n");
			out.print("Accept-Encoding: gzip\r\n");
			out.print("Accept-Language: en\r\n");
			if (cookie != null)
				{
				out.print("Cookie: "+cookie+"\r\n");
				}
			out.print("Accept-Charset: iso-8859-1,*,utf-8\r\n\r\n");

			out.print(urlEncodedData+"\r\n");
			out.flush();

			String s;
			while ((s=in.readLine()) != null)
				{
				retStr = retStr + s +"\n";
				}
			
			in.close();
			out.close();
			if (retStr.length() < 1)
				{
				// zero length results? This is an error!
				return null;
				}
			if (retStr.indexOf("HTTP/1.0 302") != -1)
				{
				// redirect! Handle it before returning.
				// whatever.
				Vector newCookies = new Vector();
				String dest="";
				StringTokenizer tokenizer=new StringTokenizer(retStr,"\n");
				while (tokenizer.hasMoreTokens())
					{
					String tok = tokenizer.nextToken();
					if (tok.indexOf(":") == -1)
						{
						// not a tagged line; ignore it.
						continue;
						}
					if (tok.substring(0,tok.indexOf(":")).equalsIgnoreCase("Set-Cookie"))
						{
						newCookies.addElement(tok.substring(tok.indexOf(": ")+2));
						}
					else if (tok.substring(0,tok.indexOf(":")).equalsIgnoreCase("Location"))
						{
						dest = tok.substring(tok.indexOf(": ")+2);
						}
					}
				return getHTMLPageRaw(dest,newCookies,0);
				}
			return retStr;
			}
		catch (Exception e)
			{
			e.printStackTrace();
			retStr = "httpPost Exception! POST "+urlString+ " caused exception "+e.getMessage();
			}
		return retStr;
		}
}
