/***************************************************************************
Runtime Metric Collection Utility Class.
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.utils;

import java.util.*;

/** 	The Metrics class allows you to have a collection of named
RunMetrics, accessible via a single object. It will also generate a
global report on the states of all RunMetrics.

@see RunMetrics
@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: Metrics.java,v 1.1 2000/05/16 22:45:10 samuelg Exp $
*/
public class Metrics {

	private Hashtable theMetrics;
	
	public Metrics()
		{
		theMetrics = new Hashtable();
		}

	/** Add a RunMetric to the collection */
	public void addMetric(String metricName)
		{
		theMetrics.put(metricName, new RunMetrics(metricName));
		}

	/** Starts a named RunMetric's timing cycle */
	synchronized public void start(String metricName)
		{
		if (!theMetrics.containsKey(metricName))
			{
			addMetric(metricName);
			}
		((RunMetrics)theMetrics.get(metricName)).start();
		}

	/** Stops a named RunMetric's timing cycle */
	synchronized public void stop(String metricName)
		{
		if (!theMetrics.containsKey(metricName))
			{
			Logger.debug(Logger.ERROR,"Can't stop non-existant metric "+metricName);
			}
		else
			{
			((RunMetrics)theMetrics.get(metricName)).stop();
			}
		}

	/** generate an HTML table of all RunMetric's reports. */
	public String showAllMetrics()
		{
		StringBuffer sb = new StringBuffer();
		sb.append("<TABLE CELLSPACING=\"2\" CELLPADDING=\"2\" WIDTH=\"98%\"><TR>");
		sb.append("<TD COLSPAN=\"2\" ALIGN=\"CENTER\">");
		sb.append("<B>Overall Run Metrics</B></TD></TR><TR><TD COLSPAN=\"2\" ALIGN=\"CENTER\">");
		sb.append(systemMetrics());
		sb.append("</TD></TR>");
		Enumeration e = theMetrics.keys();
		boolean first = true;
		int count=0;
		while (e.hasMoreElements())
			{
			if (first)
				{
				sb.append("<TR><TD VALIGN=\"TOP\">");
				}
			RunMetrics r = (RunMetrics)theMetrics.get(e.nextElement());
			sb.append(r.reportHTML());
			if (first)
				{
				sb.append("</TD><TD VALIGN=\"TOP\">");
				}
			else
				{
				sb.append("</TD></TR>");
				}
			first = !first;
			}
		// reversed now!
		if (first)
			{
			sb.append("</TABLE>");
			}
		else
			{
			sb.append("</TD><TD>&nbsp;</TD></TR></TABLE>");
			}
		return sb.toString();
		}

	/** Get an HTML-formatted report from a named RunMetric */
	public String showMetrics(String metricName)
		{
		if (!theMetrics.containsKey(metricName))
			{
			Logger.debug(Logger.ERROR,"Can't report from non-existant metric "+metricName);
			return("Unknown metric "+metricName);
			}
		else
			{
			return systemMetrics()+((RunMetrics)theMetrics.get(metricName)).reportHTML();
			}
		}
		
	/** Generate an HTML table describing global system metrics */
	public String systemMetrics()
		{
		return "<TABLE WIDTH=\"100%\" BGCOLOR=\"#a0b8c8\"><TR><TD COLSPAN=\"2\"><B>System Metrics</B>"+
			"</TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Total VM memory:</TD><TD>"+
			Runtime.getRuntime().totalMemory()+
			"</TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Free VM memory:</TD><TD>"+
			Runtime.getRuntime().freeMemory()+
			"</TD></TR>\n<TR><TD VALIGN=\"RIGHT\">Threads running:</TD><TD>"+
			Thread.currentThread().getThreadGroup().activeCount()+
			"</TD></TR></TABLE>";
			

		}

}
