/***************************************************************************
Parameter Vector Utility Class.
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.utils;

import java.util.*;

/** 	The ParameterVector class takes a single, space-delimited set
of parameters from a String and wraps them in a Vector.

@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: ParameterVector.java,v 1.1 2000/05/16 22:45:10 samuelg Exp $
*/
public class ParameterVector extends java.util.Vector {

	public ParameterVector(String parameterString)
		{
		int index=0;
		StringTokenizer st = new StringTokenizer(parameterString);
		while (st.hasMoreElements())
			{
			insertElementAt(st.nextElement(),index);
			index++;
			}
		}
		
	/** @return all paramters as a space-delimited String */
	public String toSingleLine()
		{
		StringBuffer sb = new StringBuffer();
		for (int i=0;i<size();i++)
			{
			sb.append((String)elementAt(i));
			sb.append(" ");
			}
			
		return sb.toString();
		}
		
	/** Returns the parameter at the specified position,
	cast as a String. One of three identical methods.
	@see paramAt
	@see parameterAt
	*/
	public String stringAt(int i)
		{
		return (String)super.elementAt(i);
		}

	/** Returns the parameter at the specified position,
	cast as a String. One of three identical methods.
	@see stringAt
	@see paramAt
	*/
	public String parameterAt(int i)
		{
		return stringAt(i);
		}

	/** Returns the parameter at the specified position,
	cast as a String. One of three identical methods.
	@see stringAt
	@see parameterAt
	*/
	public String paramAt(int i)
		{
		return stringAt(i);
		}

}
