/***************************************************************************
String Utility Functions.
Copyright (C) 2000, Samuel Goldstein (samuelg@andthehorseyourodeinon.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

http://www.gnu.org/copyleft/gpl.html
***************************************************************************/
package com.andthehorseyourodeinon.utils;

import java.lang.*;
import java.util.*;
import java.text.DateFormat;
import java.sql.Date;
import java.net.*;
import java.io.*;
import java.text.DecimalFormat;

/** 	StringUtils is a collection of static functions for String manipulation,
formatting, and creation.
@author <A HREF="mailto:samuelg@andthehorseyourodeinon.com">SjG</A>
@author http://www.andthehorseyourodeinon.com/
@version $Id: StringUtils.java,v 1.1 2000/05/16 22:45:10 samuelg Exp $
*/
public class StringUtils {

	/** 	indexOfIgnoreCase is an analog to the indexOf method for the String class.
	@param source String to be searched
	@param searchString the String to be found within src
	@return integer position of the first match
	*/
	public static int indexOfIgnoreCase(String source, String searchString)
		{
		// return indexOfIgnoreCase(source, searchString, 0);
		return indexOfIgnoreCase(source, searchString, 0, 0);
		}

	/** 	indexOfIgnoreCase is an analog to the indexOf method for the String class.
	@param source String to be searched
	@param searchString the String to be found within src
	@param startPoint index into src where the search will begin.
	@return integer position of the first match
	*/
	public static int indexOfIgnoreCase(String source, String searchString, int startPoint)
		{
		return indexOfIgnoreCase(source, searchString, startPoint, 0);
/*		int start = startPoint;
		int end = source.length();
		
		if ((startPoint < 0) || (startPoint >= end))
			{
			return -1;
			}

		int substart = 0;
		int subend = searchString.length();
		int checkpoint = -1;	
		char c;
		boolean notFound = true;
		while ((start < end) && (notFound) && (substart + subend <= end))
			{
			c = Character.toLowerCase(source.charAt(start));
			if (substart == subend)
				{
				notFound = false;
				}
			else
				{
				if ( c == Character.toLowerCase(searchString.charAt(substart)))
					{
					if (substart == 0)
						{
						checkpoint = start;
						}
					substart++;
					}
				else
					{
					substart = 0;
					checkpoint = -1;
					}
				}
			start++;
			}
			
		return checkpoint;
*/
		}

	/** 	indexOfIgnoreCase is an analog to the indexOf method for the String class.
	@param source String to be searched
	@param searchString the String to be found within src
	@param startPoint index into src where the search will begin.
	@param maxRange the maximum distance from startPoint to look for a match
	@return integer position of the first match
	*/
	public static int indexOfIgnoreCase(String source, String searchString, int startPoint, int maxRange)
		{
		int start = startPoint;
		int end = source.length();
		
		if (maxRange > 0)
			{
			end = start + maxRange + searchString.length();
			}
		
		if ((startPoint < 0) || (startPoint >= end))
			{
			return -1;
			}

		int substart = 0;
		int subend = searchString.length();
		int checkpoint = -1;	
		char c;
		boolean notFound = true;
		while ((start < end) && (notFound) && (substart + subend <= end))
			{
			c = Character.toLowerCase(source.charAt(start));
			if (substart == subend)
				{
				notFound = false;
				}
			else
				{
				if ( c == Character.toLowerCase(searchString.charAt(substart)))
					{
					if (substart == 0)
						{
						checkpoint = start;
						}
					substart++;
					}
				else
					{
					substart = 0;
					checkpoint = -1;
					}
				}
			start++;
			}
			
		return checkpoint;
		}


	/** 	Determines whether a string contains any characters that are
	not either alphanumeric or period, underscore, space, or hyphen.
	@param theString String to be searched
	*/
	public static boolean isAlphaSafeString(String theString)
		{
		if (theString == null)
			{
			return true;
			}
		int i=0;
		while (i<theString.length())
			{
			if (!Character.isLetterOrDigit(theString.charAt(i)))
				{
				char c = theString.charAt(i);
				if (!((c == '.') || (c == '-') || (c == '_') ||
					(c == ' ')))
						{
						return false;
						}
				}
			i++;
			}
		return true;
		}

	/** 	Determines whether a string contains any characters that are
	not alphanumeric.
	@param theString String to be searched
	*/
	public static boolean isAlphaNumeric(String theString)
		{
		if (theString == null)
			{
			return true;
			}
		int i=0;
		while (i<theString.length())
			{
			if (!Character.isLetterOrDigit(theString.charAt(i)))
				{
				return false;
				}
			i++;
			}
		return true;
		}
		
	/** 	Given a string, this will replace all instances of a given substring with
	another given string.
	@param original String to be modified.
	@param find the String to be replaced.
	@param replace the String to replace with.
	@return a string with the completed substitutions.
	*/
	public static String replaceSubstring(String original, String find,
			String replace)
		{
		String retStr = new String(original);
		int startIndex = 0;
		int endpt;
	
		while (retStr.indexOf(find,startIndex) > -1)
			{
			endpt = retStr.indexOf(find,startIndex);
			retStr = retStr.substring(0,endpt) +
				replace +
				retStr.substring(endpt + find.length());
			startIndex = endpt + replace.length();
			}
        
		return retStr;
		}

	/** 	Given a string, this will replace all instances of a given substring with
	another given string. This will replace instances without regard to case.
	@param original String to be modified.
	@param find the String to be replaced.
	@param replace the String to replace with.
	@return a string with the completed substitutions.
	*/
	public static String replaceSubstringIgnoreCase(String original, String find,
			String replace)
		{
		String retStr = new String(original);
		int startIndex = 0;
		int endpt;
	
		while (indexOfIgnoreCase(retStr,find,startIndex) > -1)
			{
			endpt = indexOfIgnoreCase(retStr, find,startIndex);
			retStr = retStr.substring(0,endpt) +
				replace +
				retStr.substring(endpt + find.length());
			startIndex = endpt + replace.length();
			}
        
		return retStr;
		}

	/** 	Given a string containing markup tags, this function will separate out all
	elements between a given start and end tag. It will return these elements
	as a Vector of Strings. Tags are not case sensitive.
	@param source String to be processed.
	@param startTag String representing the tag indicating the start of an element.
	@param endTag String representing the tag indicating the end of an element.
	@return a Vector of Strings.
	*/
	public static Vector betweenTags(String source, String startTag, String endTag)
		{
		Vector theRet = new Vector();
		int start, end, realEnd;
		boolean more;
		
		realEnd = indexOfIgnoreCase(source, endTag);
		
		if (realEnd == -1)
			{
			return theRet;
			}
			
		more = true;
		start = 0;
		end = 0;
		while (more)
			{
			start = indexOfIgnoreCase(source, startTag, end);
			end = indexOfIgnoreCase(source, endTag, start+startTag.length());
			if ((start < 0) || (end < 0))
				{
				more = false;
				}
			else
				{
				theRet.addElement(source.substring(start+startTag.length(),end));
				}
			if (end == realEnd)
				{
				more = false;
				}
			}
			
		return theRet;
		}

	/** 	Given a string containing markup tags, this function will return the first
	element between a given start and end tag. It will return this element as a
	String. Tags are not case sensitive.
	@param source String to be processed.
	@param startTag String representing the tag indicating the start of an element.
	@param endTag String representing the tag indicating the end of an element.
	@return a Vector of Strings.
	*/
	public static String singleBetweenTags(String source, String startTag, String endTag)
		{
		int start, end;
		start = indexOfIgnoreCase(source, startTag);
		end = indexOfIgnoreCase(source, endTag,start+startTag.length());
		
		if ((start == -1) || (end == -1))
			{
			return source;
			}
			
		return source.substring(start+startTag.length(),end);
		}
		

	/** 	Given a string of HTML or other text containing URLs, this rather simplistic function
	will attempt to extract the URLs, which it will return as a Vector of
	Strings. It only finds URLs that include the HTTP method (<I>i.e.,</I> "http://"),
	and it does not return this method with the URLs it extracts.
	@param source String to be processed.
	@return a Vector of Strings.
	*/
	public static Vector extractURLs(String source)
		{
		Vector theRet = new Vector();
		
		int start, start1, end;
		
		end = source.length();
		start = 0;
		
		while (start < end)
			{
			start1 = end;
			if (indexOfIgnoreCase(source, "http", start) != -1)
				{
				start1 = indexOfIgnoreCase(source, "http", start);
				// start1 points at the "h" of "http". We want to allow
				// for a fair amount of variation on the part of the HTML,
				// so given the possibilities:
				// href=http...      href="http...      href= "http...
				// href ="http...    href = http...     href = "http...
				// we go with the longest, and play from there...
				// Note: we could miss a ref if it's the very first thing in a file!				
				if (start1 >= 8)
					{
					int check = indexOfIgnoreCase(source, "href", start1 - 8, 8);
					if ((check == -1) || (check >= start1))
						{
						// this ain't a href="http... kinda reference!
						start = start1 + 1;
						continue; 
						}
					}
				}
			else
				{
				// no more!
				start = end + 1;
				}

			start = source.indexOf("://",start1);
			
			if (start == -1)
				{
				start = start1 + 1;
				continue;
				}
			start += "://".length();
			if (indexOfIgnoreCase(source,"services.google",start,"services.google".length()) == start)
				{
				// another googlism to ignore...
				continue;
				}
			if (indexOfIgnoreCase(source,"directory.google",start,"directory.google".length()) == start)
				{
				// another googlism to ignore...
				continue;
				}
			if (indexOfIgnoreCase(source,"www.google",start,"www.google".length()) == start)
				{
				// another googlism to ignore...
				continue;
				}
			if (indexOfIgnoreCase(source,"jump.altavista",start,"jump.altavista".length()) == start)
				{
				// altavistaism to ignore...
				continue;
				}
			if (indexOfIgnoreCase(source,"solutions.altavista",start,"solutions.altavista".length()) == start)
				{
				// another altavistaism to ignore...
				continue;
				}
			if (indexOfIgnoreCase(source,"www.altavista",start,"www.altavista".length()) == start)
				{
				// another altavistaism to ignore...
				continue;
				}
			StringBuffer newUrl = new StringBuffer();
			char c = source.charAt(start);
			while ((start < end) && (c != ' ') && (c != '>') && (c != '"') && (c != '\n') &&
				(c != '\r') && (c != ':'))
				{
				newUrl.append(c);
				start++;
				c = source.charAt(start);
				}
			if (start != end)
				{
				theRet.addElement(newUrl.toString());
				}
			}

		return theRet;
		}

	/** 	Given a string of HTML or other text containing URLs, this rather simplistic function
	will attempt to extract the URLs, which it will return as a Vector of
	Strings. It only finds URLs that include the HTTP method (<I>i.e.,</I> "http://"),
	and it does not return this method with the URLs it extracts.
	@param source String to be processed.
	@return a Vector of Strings.
	*/
	public static Vector oldExtractURLs(String source)
		{
		Vector theRet = new Vector();
		
		int start, start1, end;
		
		end = source.length();
		start = 0;
		
		while ((start < end) && (indexOfIgnoreCase(source, "http", start) != -1))
			{
			/* Below, there's some funky looking code to
			   deal with some search-engine-specific cases:
			   Google wraps stuff in a RealName url sometimes,
			          so we need to get rid of any q=http... entries
			*/
			start1 = end;
			if (indexOfIgnoreCase(source, "http", start) != -1)
				{
				start1 = indexOfIgnoreCase(source, "http", start);
				if ((start1 >= 3) &&
					((source.substring(start1 - 3,start1).equalsIgnoreCase("q=\"")) ||
					 (source.substring(start1 - 2,start1).equalsIgnoreCase("q="))))
					{
					// this is a googlism... ignore this one.
					start = start1 + 1;
					continue;
					}
				if ((start1 >= 5) &&
					(source.substring(start1 - 5,start1).equalsIgnoreCase("d%3f")))
					{
					// this is a googlism... ignore this one.
					start = start1 + 1;
					continue;
					}
				}

			start = source.indexOf("://",start1);
			
			if (start == -1)
				{
				start = start1;
				continue;
				}
			start += "://".length();
			if (indexOfIgnoreCase(source,"services.google",start) == start)
				{
				// another googlism to ignore...
				continue;
				}
			if (indexOfIgnoreCase(source,"directory.google",start) == start)
				{
				// another googlism to ignore...
				continue;
				}
			StringBuffer newUrl = new StringBuffer();
			char c = source.charAt(start);
			while ((start < end) && (c != ' ') && (c != '>') && (c != '"') && (c != '\n') &&
				(c != '\r') && (c != ':'))
				{
				newUrl.append(c);
				start++;
				c = source.charAt(start);
				}
			if (start != end)
				{
				theRet.addElement(newUrl.toString());
				}
			}

		return theRet;
		}


	/** 	Given a string of HTML or other text containing URLs, this brutally simplistic function
	will singlemindedly strip out anything between &lt; and &gt; marks. It uses no
	intelligence to determine whether or not these have anything to
	do with tags whatsoever.
	@param source String to be processed.
	@return a String without tags (and possibly without equations, if they were there).
	*/
	public static String blowAwayTags(String source)
		{
		String retStr = new String();
		boolean inTag = false;

		int i=0;
		while (i<source.length())
			{
			if ((source.charAt(i) == '<') && (source.indexOf(">",i) != -1))
				{
				i = source.indexOf(">",i);
				}
			else
				{
				retStr = retStr + source.charAt(i);
				}
			i++;
			}

		return retStr;
		}


	/** 	Given a string and a length, this will truncate the String to fit
	within that character length, adding ellipsis to indicate that the
	String has been shortened. Ellipsis will only be added at word breaks.
	@param source the String to be truncated.
	@param maxLen the maximum number of characters for the resultant String
	@return a truncated String.
	*/
	public static String truncateNicely(String source, int maxLen)
		{
		int chopPoint;

		if (source.length() < maxLen)
			{
			return source;
			}
			
		chopPoint = source.lastIndexOf(" ",maxLen);
		if (chopPoint < 0)
			{
			return source.substring(0,maxLen);
			}
		return source.substring(0,chopPoint)+"...";
		}

	/** 	Converts millisecond sinto a human-readable
	String of days, hours, minutes, seconds, and
	milliseconds.
	@param ms number of milliseconds
	@return human-readable String
	*/
	public static String reasonableTimeUnits(long ms)
		{
		return displayTimeUnits(ms, false);
		}

	/** 	Converts millisecond sinto a human-readable
	String of days, hours, minutes, seconds, and
	milliseconds. There is an option to abbreviate the String; this would
	result in days being labeled with a "d" rather than the word "days," etc.
	@param ms number of milliseconds
	@param abbreviated boolean telling whether to use long or short labels.
	@return human-readable String
	*/
	public static String displayTimeUnits(long ms, boolean abbreviated)
		{
		// where ms is milliseconds, of course.
		DecimalFormat fnFmt = new DecimalFormat("###0.###");
		DecimalFormat msFmt = new DecimalFormat("####.###");
		DecimalFormat noFmt = new DecimalFormat("#####");
		String response = "";
		long tmp;
		double tmp2;

		if (ms >= 86400000)
			{
			tmp = ms/86400000;
			response = response + noFmt.format(tmp);
			
			if (abbreviated)
				{
				response = response + "d ";
				}
			else
				{
			
				if (tmp > 1)
					{
					response = response + " days ";
					}
				else
					{
					response = response + " day ";
					}
				}
			ms = ms - (86400000*(int)tmp);
			}

		if (ms >= 3600000)
			{
			tmp = ms/3600000;
			response = response + noFmt.format(tmp);
			if (abbreviated)
				{
				response = response + "h ";
				}
			else
				{
				if (tmp > 1)
					{
					response = response + " hours ";
					}
				else
					{
					response = response + " hour ";
					}
				}
			
			ms = ms - (3600000*(int)tmp);
			}
			
		if (ms >= 60000)
			{
			tmp = ms/60000;
			response = response + noFmt.format(tmp);
			if (abbreviated)
				{
				response = response + "m ";
				}
			else
				{
				if (tmp > 1)
					{
					response = response + " minutes ";
					}
				else
					{
					response = response + " minute ";
					}
				}
			
			ms = ms - (60000*(int)tmp);
			}
			
		tmp2 = ms/1000.0;
		response = response + fnFmt.format(tmp2);
		if (abbreviated)
			{
			response = response + "s";
			}
		else
			{
			response = response + " seconds ";
			}

		return response;
		}

	/** 	This Function Will Convert A String To Start Each Word With An Upper Case
	Character.
	@param st String To Be Converted
	@return A Mixed Case String
	@author Karl Zilles
	*/	
	public static String toMixedCase(String st)
		{
		StringBuffer result = new StringBuffer(st.length());
		boolean startChar = true;
		for (int i=0; i<st.length(); i++)
			{
			char ch = st.charAt(i);
			result.append( startChar ? Character.toUpperCase(ch) : Character.toLowerCase(ch));
			startChar = !Character.isLetterOrDigit(ch);
			}
		return result.toString();
		}
	/** 	Create a date/time based file spec using the current System time and date.<BR>
	Example: timestampFilespec("log","txt")<BR>
	returns: log20000505-101501.txt
	@param prefix String with which to prefix the filespec
	@param extension String to use as a file-type extension. Should not include the dot.
	@return A filespec.
	*/	
	public static String timestampFilespec(String prefix, String extension)
		{
		return prefix+createCompactDateString("","-","")+"."+extension;
		}

	/** 	Return a String containing a compact date/time using the current System time and date.
	@return A String of the form "2000-05-05 10:15:01".
	*/	
	public static String createCompactDateString()
		{
		Calendar now = Calendar.getInstance();
		return createCompactDateString(now, "-", " ", ":");
		}

	/** 	Return a String containing a compact date/time using the time and date
	provided.
	@param when the Calendar object to use for creating the String.
	@return A String of the form "2000-05-05 10:15:01".
	*/	
	public static String createCompactDateString(Calendar when)
		{
		return createCompactDateString(when, "-", " ", ":");
		}

	/** 	Return a String containing a compact date/time using the current System time and date, with
	user-specified delimiters.
	@param yearDelimiter the String to use to separate the digits representing year, month, and day.
	@param splitDelimiter the String to use to separate the digits representing date and time.
	@param hourDelimiter the String to use to separate the digits representing hour, minute, and second.
	@return A String containing the date using the specified delimiters.
	*/	
	public static String createCompactDateString(String yearDelimiter, String splitDelimiter, String hourDelimiter)
		{
		Calendar now = Calendar.getInstance();
		return createCompactDateString(now, yearDelimiter, splitDelimiter, hourDelimiter);
		}


	/** 	Return a String containing a compact date/time using the specified time and date, with
	user-specified delimiters.
	@param when the Calendar object to use for creating the String.
	@param yearDelimiter the String to use to separate the digits representing year, month, and day.
	@param splitDelimiter the String to use to separate the digits representing date and time.
	@param hourDelimiter the String to use to separate the digits representing hour, minute, and second.
	@return A String containing the date using the specified delimiters.
	*/	
	public static String createCompactDateString(Calendar when, String yearDelimiter, String splitDelimiter,
		String hourDelimiter)
		{
		StringBuffer prefix = new StringBuffer();
		prefix.append(when.get(Calendar.YEAR) + yearDelimiter);
		if (when.get(Calendar.MONTH) < 9)
			{
			prefix.append("0");
			}
		prefix.append((when.get(Calendar.MONTH)+1)+yearDelimiter);
		if (when.get(Calendar.DAY_OF_MONTH) < 10)
			{
			prefix.append("0");
			}
		prefix.append(when.get(Calendar.DAY_OF_MONTH)+splitDelimiter);
		if (when.get(Calendar.HOUR_OF_DAY) < 10)
			{
			prefix.append("0");
			}
		prefix.append(when.get(Calendar.HOUR_OF_DAY)+hourDelimiter);
		if (when.get(Calendar.MINUTE) < 10)
			{
			prefix.append("0");
			}
		prefix.append(when.get(Calendar.MINUTE)+hourDelimiter);
		if (when.get(Calendar.SECOND) < 10)
			{
			prefix.append("0");
			}
		prefix.append(when.get(Calendar.SECOND));
		return prefix.toString();
		}
		
	/** 	Return a String containing an RFC 1123-compliant date String (RFC 1123 supercedes the date
	specifications of RFC 822) using the current System time and date.
	@return A String containing the date.
	*/	
	public static String createRFC1123DateString()
		{
		Calendar now = Calendar.getInstance();
		return toRFC1123DateString(now);
		}

	/** 	Return a String containing an RFC 1123-compliant date String (RFC 1123 supercedes the date
	specifications of RFC 822) using the specified time and date.
	@param when the Calendar object to use for creating the String.
	@return A String containing the date.
	*/	
	public static String toRFC1123DateString(Calendar when)
		{
		StringBuffer sb = new StringBuffer();
		String dayOfWeek[]={"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
		String monthOfYear[]={"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
		sb.append(dayOfWeek[when.get(Calendar.DAY_OF_WEEK)-1]);
		sb.append(", "+when.get(Calendar.DAY_OF_MONTH));
		sb.append(" "+monthOfYear[when.get(Calendar.MONTH)-1]);
		sb.append(" "+when.get(Calendar.YEAR));
		sb.append(" ");
		if (when.get(Calendar.HOUR_OF_DAY)<10)
			{
			sb.append("0");
			}
		sb.append(when.get(Calendar.HOUR_OF_DAY)+":");
		if (when.get(Calendar.MINUTE)<10)
			{
			sb.append("0");
			}
		sb.append(when.get(Calendar.MINUTE)+":");
		if (when.get(Calendar.SECOND)<10)
			{
			sb.append("0");
			}
		sb.append(when.get(Calendar.SECOND));			
		sb.append(" "+((TimeZone)when.getTimeZone()).getID());
		return sb.toString();
		}
}
